@@ -47,7 +47,8 @@ sub handleTarget {
     my(@inst);
     if (defined($target->{'inst'})) {
         my($inst) = $target->{'inst'};
-        $inst = ConfigTree::Cache::expandString($inst, $target, \&Warn);
+        $inst = ConfigTree::Cache::expandString($inst, $target, \&Warn)
+                    if (length($inst) > 2 && index($inst, "%") >= 0);
 
         Debug("Evaling inst which is: $inst");
         $inst = quoteString($inst);
@@ -124,11 +125,13 @@ sub checkTargetInstance {
 
     my($m) = new Monitor;
     my($rrd) = new RRD::File( -file=>$target->{'rrd-datafile'} );
-    if (!$rrd->open() || !$rrd->loadHeader()) {
+    if (!$rrd->open()) {
         Info("Couldn't open RRD File for $name, skipping this target.");
         return;
     }
 
+    my $rrdpollinterval = $target->{'rrd-poll-interval'};
+    my $time = time();
     # Convert backslashed commas to \0's ;
     my $ThresholdString = $target->{'monitor-thresholds'};
     $ThresholdString =~ s/\\,/\0/g ;
@@ -137,24 +140,36 @@ sub checkTargetInstance {
     my($Threshold);
     # General monitor threshold format:
     # datasource:monitor type:monitor args:action:action args
-    # Current supported actions: FUNC, EXEC, FILE, MAIL, SNMP
+    # Current supported actions: FUNC, EXEC, FILE, MAIL, SNMP, META
     # Action args are colon (:) delimited.
     # Monitor args are colon (:) delimited, but must not collide with
     # any of the action tags.
     # SNMP is the exception; it has no action args and instead uses
     # the target variable trap-address. This exception is maintained for
     # backwards compatibility.
+    # META only stores an alarm in the *.meta files; It can have any number of
+    # action arguments which can serve to group alarms
+    # for classification or to identify a priority priority level. Arguments
+    # are not interpreted or processed in any way by Monitor.pm.
     foreach $Threshold (@ThresholdStrings) {
         # restore escaped commas
         $Threshold =~ s/\0/,/g ;
+        my($span) = 0;
+        my($spanlength);
+
         my($ds,$type,$args) = split(/\s*:\s*/, $Threshold, 3);
+        ($args, $spanlength) = split(/:SPAN:/, $args, 2) if ($args =~ /SPAN/);
+        if (defined ($spanlength) && $spanlength =~ /\d+/ ){
+            $span = 1;
+        }
+
         # hide escaped colons
         $args =~ s/\\:/\0/g ;
         # default action type is SNMP
         my($actionType) = 'SNMP' ;
         my(@actionArgs);
         # search for an action tag
-        if ( $args =~ /^(.*)\s*:\s*(FUNC|EXEC|FILE|MAIL)\s*:\s*(.*)$/ ) {
+        if ( $args =~ /^(.*)\s*:\s*(FUNC|EXEC|FILE|MAIL|META)\s*:\s*(.*)$/ ) {
             $args = $1 ;
             $actionType = $2 ;
             # restore escaped colons in the monitor args field
@@ -164,8 +179,9 @@ sub checkTargetInstance {
             @actionArgs = split(/\s*:\s*/, $action_args);
             # restore escaped colons in the action args field
             map { $_ =~ s/\0/:/g } @actionArgs;
-        } elsif ( $args =~ /^(.*)\s*:\s*SNMP\s*$/ ) {
+        } elsif ( $args =~ /^(.*)\s*:\s*(SNMP|META)\s*$/ ) {
             $args = $1 ;
+            $actionType = $2 ;
             # restore escaped colons
             $args =~ s/\0/:/g ;
         }
@@ -191,13 +207,30 @@ sub checkTargetInstance {
                 }
             } else {
                 my($metaRef) = $rrd->getMeta();
-                LogMonitor("$name - $Threshold failed.");
-                if ($persistent eq "true" || !defined($metaRef->{$Threshold})) {
+                my ($x, $mt, $mevent, $rest);
+                ($x,$mt, $mevent, $rest) = split (/\s+/, $metaRef->{$Threshold},4)
+                                                 unless (!defined($metaRef->{$Threshold}));
+                $mt = $time if !defined($mt);
+                LogMonitor("$name -  $mt - $Threshold failed.");
+                my($spanfail) = 0;
+
+                if ($span && $mt < ($time - ($spanlength * $rrdpollinterval) - 90)) {
+                        $spanfail = 1;
+                }
+                if ((!$span || $span && $spanfail) &&
+                    ($persistent eq "true" || !defined($metaRef->{$Threshold})) ) {
+
                     LogMonitor("Triggering alarm for $Threshold.");
                     $m->Alarm($target,$name,$ds,$type,$Threshold,$actionType,\@actionArgs,$val);
-                    $metaRef->{$Threshold} = 'Failed';
+                    $metaRef->{$Threshold} = ' '. $mt .' failure value '. $val;
                     $rrd->setMeta($metaRef);
-                }
+                } elsif ($span && (!defined($mevent)) || (($mevent ne 'failure') && $spanfail))  {
+                    LogMonitor("Triggering a span event for $Threshold.");
+                    my($event) = ($spanfail) ? 'failure' : 'spanevent';
+                    $m->Alarm($target,$name,$ds,$type,$Threshold,$actionType,\@actionArgs,$val) if ($spanfail);
+                    $metaRef->{$Threshold} = ' '. $mt . ' ' . $event . ' value '. $val;
+                    $rrd->setMeta($metaRef);
+               }
             }
         } else {
             Warn("No monitor handler defined for monitor type $type");
