@@ -22,6 +22,7 @@ use strict;
 use snmpUtils;
 use RRD::File;
 use Common::Log;
+use Common::Util;
 
 # Registered monitors
 $Common::global::gMonitorTable{'value'} = \&monValue;
@@ -32,6 +33,14 @@ $Common::global::gMonitorTable{'exact'} = \&monExact;
 $Common::global::gMonitorTable{'failures'} = \&monFailures;
 $Common::global::gMonitorTable{'quotient'} = \&monQuotient;
 
+# NaN Alarm processing
+# Return value of nanErr = 1 is not an alarm or threshold pass
+# Return value of nanErr = 0 is an alarm or threshold failure
+my $nanErr = 1;
+if ($Common::global::gEnableNoValueAlarms) {
+    $nanErr = ($Common::global::gEnableNoValueAlarms) ? 0 : 1;
+}
+
 sub new {
     my($package) = @_;
     my $self = { };
@@ -50,14 +59,12 @@ sub monValue {
                                  );
 
     if (!defined($value)) {
-        Warn("Skipping: Couldn't fetch last value from datafile.");
+        Warn("Monitor: Couldn't fetch last $ds value from " .
+             $target->{'rrd-datafile'}.".");
         return 1;
     }
 
-    if ($value =~ /^NaN/)  {
-        Info("Skipping: Last value from datafile was NaN.");
-        return 1;
-    }
+    return ($nanErr, 'NaN') if isNaN($value);
 
     $min = shift(@Thresholds);
     $min = 'n' if (! defined($min));
@@ -65,7 +72,7 @@ sub monValue {
     if (lc($min) eq 'n') {
         $minOK = 1;
     } else {
-        $minOK = ($value > $min);
+        $minOK = ($value > $min) ? 1 : 0;
     }
 
     $max = shift(@Thresholds);
@@ -74,10 +81,9 @@ sub monValue {
     if (lc($max) eq 'n') {
         $maxOK = 1;
     } else {
-        $maxOK = ($value < $max)
+        $maxOK = ($value < $max) ? 1 : 0;
     }
     Debug ("Value is $value; min is $min; max is $max");
-
     return ($maxOK && $minOK,$value);
 }
 
@@ -87,7 +93,8 @@ sub monHunt {
     my($roll, $cmp_name, $cmp_ds) = split(/\s*:\s*/, $args);
 
     if (!defined($roll)) {
-        Warn("Skipping: Missing rollover value in hunt threshold.");
+        Warn("Monitor: Missing rollover value in hunt threshold for " .
+             $target->{'auto-target-name'} . " datasource $ds.");
         return 1;
     }
 
@@ -96,7 +103,8 @@ sub monHunt {
                                  $self->getDSNum($target, $ds), 0);
 
     if (!defined($value)) {
-        Warn("Skipping: Couldn't fetch last value from datafile.");
+        Warn("Monitor: Couldn't fetch last $ds value from " .
+             "$target->{'rrd-datafile'}.");
         return 1;
     }
     if ($value == 0) {
@@ -114,7 +122,7 @@ sub monHunt {
                                                 $Common::global::gConfigRoot);
             ConfigTree::Cache::expandHash($cmp_target, $cmp_target, \&Warn);
         } else {
-            Warn("Skipping: No such target: $cmp_name");
+            Warn("Monitor: No such target: $cmp_name");
             return 1;
         }
     } else {
@@ -126,7 +134,7 @@ sub monHunt {
                                      $self->getDSNum($cmp_target, $cmp_ds), 0);
 
     if (!defined($cmp_value)) {
-        Warn("Skipping: Couldn't fetch current value from $cmp_name");
+        Warn("Monitor: Couldn't fetch current value from $cmp_name");
         return 1;
     }
 
@@ -147,7 +155,8 @@ sub monRelation {
     $cmp_time = 0 unless(defined($cmp_time));
 
     if (!defined($thresh) || !defined($cmp_time)) {
-        Warn("Skipping: Improperly formatted relation monitor.");
+        Warn("Monitor: Improperly formatted relation monitor for " .
+             "$target->{'auto-target-name'} datasource $ds.");
         return 1;
     }
 
@@ -164,12 +173,21 @@ sub monRelation {
                                  $self->getDSNum($target,$ds), 0);
 
     if (!defined($value)) {
-        Warn("Skipping: Couldn't fetch last value from datafile.");
+        Warn("Monitor: Couldn't fetch last $ds value from " .
+             "$target->{'rrd-datafile'}.");
         return 1;
     }
+    return ($nanErr, 'NaN') if isNaN($value);
 
-    my $cmp_value = $self -> FetchComparisonValue($target,$ds,$cmp_name,$cmp_ds,$cmp_time);
-    return 1 if ($cmp_value =~ /^NaN/);
+    my ($cmp_value);
+    # Is this a straight comparison value or should we fetch it as a DS
+    if ($cmp_ds =~ /^[-+]?([0-9]*\.[0-9]+|[0-9]+)$/) {
+        $cmp_value = $cmp_ds;
+    # Fetch the value it must be a DS
+    } else {
+        ($cmp_value) = $self -> FetchComparisonValue($target,$ds,$cmp_name,$cmp_ds,$cmp_time);
+        return ($nanErr,'NaN') if isNaN($cmp_value);
+    }
 
     my($difference) = abs($cmp_value - $value);
     $thresh = abs($thresh); # differences are always positive
@@ -179,15 +197,17 @@ sub monRelation {
         if ($cmp_value == 0) {
             #avoid division by 0
             if ($difference == 0 && $gtlt eq '<') {
-                return 1;
+                return (1,$value);
             } else {
-                return 0;
+                return (0,$value);
             }
         }
         $difference = $difference / abs($cmp_value) * 100;
     }
-    Debug("Values $value, $cmp_value; Difference is $difference; gtlt is $gtlt; thresh is $thresh.");
-    return (0,$value) if (!(eval "$difference $gtlt $thresh"));
+    Debug("Values $value, $cmp_value; Difference is $difference; " .
+          "gtlt is $gtlt; thresh is $thresh.");
+# see documentation: threshold fails if expression is false
+    return (0,$value) if ((eval "$difference $gtlt $thresh") == 0);
     return (1,$value);
 }
 
@@ -196,7 +216,8 @@ sub monExact {
     my(@args) = split(/\s*:\s*/, $args);
 
     if (@args != 1) {
-        Warn("Skipping: monitor type \"exact\" requires 1 argument.");
+        Warn("Monitor: monitor type \"exact\" requires 1 argument for " .
+             "$target->{'auto-target-name'} $ds.");
         return 1;
     }
     my $exact = shift @args;
@@ -207,9 +228,11 @@ sub monExact {
                                  );
 
     if (!defined($value)) {
-        Warn("Skipping: Couldn't fetch last value from datafile.");
+        Warn("Monitor: Couldn't fetch last $ds value from " .
+             "$target->{'rrd-datafile'}.");
         return 1;
     }
+    return ($nanErr,'NaN') if isNaN($value);
 
     return $exact eq $value ? 0 : 1;
 }
@@ -222,6 +245,14 @@ sub monExact {
 sub monFailures {
     my($self, $target, $ds, $type, $args) = @_;
 
+    # only check value range if specified, as this requires fetching from the RRD
+    if ($args =~ /:/) {
+        # monValue returns 0 if out of range, 1 otherwise
+        my ($rc, $value) = $self->monValue($target,$ds,'value',$args);
+        Debug("value is $value, in range $rc");
+        # value is out of range, so ignore any failure alarm
+        if ($rc == 0) { return 1 };
+    }
     my $datafile = $target->{'rrd-datafile'};
     my $dsNum = $self->getDSNum($target, $ds);
     return 1 if (!defined($dsNum));
@@ -248,11 +279,12 @@ sub monFailures {
     my $ret;
     $ret = $self->{openrrd}->getDSRowValue($rraNum,0,$dsNum);
     if (!defined($ret)) {
-        Warn("Skipping: Couldn't fetch last value from datafile FAILURES RRA.");
+        Warn("Monitor: Couldn't fetch last $ds value from " .
+             "$target->{'rrd-datafile'} FAILURES RRA.");
         return 1;
     }
     # FAILURES array stores a 1 for a failure (so should return 0)
-    return 1 if ($ret =~ /^NaN/);
+    return (1,'NaN') if isNaN($ret);
     return !($ret);
 }
 
@@ -268,11 +300,13 @@ sub monQuotient {
     my($thresh, $cmp_name, $cmp_ds, $cmp_time) = split(/\s*:\s*/, $args);
     my($pct) = ($thresh =~ s/\s*pct\s*//i);
     $cmp_time = 0 unless(defined($cmp_time));
-    Info("Use of quotient monitors without percent threshold is deprecated")
+    Info("Monitor: Use of quotient monitors without percent threshold is " .
+         "deprecated for $target->{'auto-target-name'} datasource $ds.")
         unless (defined($pct));
 
     if (!defined($thresh) || !defined($cmp_time)) {
-        Warn("Skipping: Improperly formatted quotient monitor.");
+        Warn("Monitor: Improperly formatted quotient monitor for " .
+             "$target->{'auto-target-name'} datasource $ds.");
         return 1;
     }
 
@@ -288,12 +322,22 @@ sub monQuotient {
                                  $self->getDSNum($target,$ds), 0);
 
     if (!defined($value)) {
-        Warn("Skipping: Couldn't fetch last value from datafile.");
+        Warn("Monitor: Couldn't fetch last $ds value from " .
+             "$target->{'rrd-datafile'}.");
         return 1;
     }
+    return ($nanErr,'NaN') if isNaN($value);
 
-    my $cmp_value = $self -> FetchComparisonValue($target,$ds,$cmp_name,$cmp_ds,$cmp_time);
-    return 1 if ($cmp_value =~ /^NaN/);
+    my ($cmp_value);
+    # Is this a straight comparison value or should we fetch it as a DS
+    if ($cmp_ds =~ /^[-+]?([0-9]*\.[0-9]+|[0-9]+)$/) {
+        $cmp_value = $cmp_ds;
+
+    # Fetch the value it must be a DS
+    } else {
+        ($cmp_value) = $self -> FetchComparisonValue($target,$ds,$cmp_name,$cmp_ds,$cmp_time);
+        return ($nanErr,'NaN') if isNaN($cmp_value);
+    }
 
     my($difference) = abs($cmp_value - $value);
     $thresh = abs($thresh); # differences are always positive
@@ -331,7 +375,7 @@ sub FetchComparisonValue {
                                                 $Common::global::gConfigRoot);
             ConfigTree::Cache::expandHash($cmp_target, $cmp_target, \&Warn);
         } else {
-            Warn("Skipping: No such target: $cmp_name");
+            Warn("Monitor: No such target: $cmp_name");
             return 'NaN';
         }
     }
@@ -343,13 +387,13 @@ sub FetchComparisonValue {
                                      $cmp_time);
 
     if (!defined($cmp_value)) {
-        Warn("Skipping: Couldn't fetch value for $cmp_time ".
+        Warn("Monitor: Couldn't fetch value for $cmp_time ".
              "seconds ago from $cmp_name.");
         return 'NaN';
     }
 
-    if ($cmp_value =~ /NaN/) {
-        Info("Skipping: Data for $cmp_time seconds ago from " .
+    if (isNaN($cmp_value)) {
+        Info("Monitor: Data for $cmp_time seconds ago from ".
              "$cmp_name is NaN");
         return 'NaN';
     }
@@ -388,13 +432,13 @@ sub rrdFetch {
     }
 
     if ($lastrecord < $sec) {
-        Warn("Fetch Failed: RRD file does not have data going back " .
+        Warn("Monitor: $datafile does not have data going back " .
              "$sec seconds.");
         return;
     }
 
     if ($sec % ($lastrecord / $rra->{row_cnt})) {
-        Warn("Fetch Failed: RRA required to find data $sec seconds ago " .
+        Warn("Monitor: RRA required to find data $sec seconds ago " .
              "is too granular.");
         return;
     }
@@ -427,95 +471,153 @@ sub getDSNum {
 
 # Subroutines to handle alarms
 
+sub dispatchAlarm {
+# order of arguments: $self, $target, $name, $ds, $type, $threshold,
+#                     $alarmType, $alarmArgs, $val
+    my ($args, $action) = @_;
+
+    my ($target, $ds, $val) = ($$args[1], $$args[3], $$args[8]);
+
+    my $alarmType       = $$args[6];
+
+    my %dispatch = (
+        EXEC => \&alarmExec,
+        FILE => \&alarmFile,
+        FUNC => \&alarmFunc,
+        MAIL => \&alarmMail,
+        SNMP => \&alarmSnmp,
+        META => \&alarmMeta,
+    );
+
+    my $alarm = $dispatch{$alarmType};
+
+    unless (defined($alarm)) {
+        Warn("Monitor: unknown alarm $alarmType for target: ".
+             $target->{'auto-target-path'} . $target->{'auto-target-name'} .
+             " for datasource $ds");
+        return;
+    }
+
+    my $return = $alarm->($args, $action);
+    return;
+};
+
+
 # Process alarm action
 sub Alarm {
-    my($self,$target,$name,$ds,$type,$threshold,$alarmType,$alarmArgs,$val) = @_;
+# order of arguments: $self, $target, $name, $ds, $type, $threshold,
+#                     $alarmType, $alarmArgs, $val
+    my $action = 'ADD';
 
-    if ($alarmType eq 'EXEC') {
-        system($alarmArgs->[0]);
-        Info("Triggered event with shell command '".$alarmArgs->[0]."' .");
-    } elsif ($alarmType eq 'MAIL') {
-        $self->sendEmail(
-                         'alarm',
-                         $alarmArgs,
-                         $type,
-                         $threshold,
-                         $name,
-                         $ds,
-                         $val,
-                         $target->{'inst'});
-    } elsif ($alarmType eq 'FUNC') {
-        if (defined $main::gMonFuncEnabled) {
-            eval($alarmArgs->[0]);
-            Info("Triggered event with FUNC '".$alarmArgs->[0]."' .");
-        } else {
-            Warn("Exec monitor $threshold triggered, but executable alarms are not enabled.");
-        }
-    } elsif ($alarmType eq 'SNMP') {
-        my($Specific_Trap_Type) = 4; # Violation Trap
-        $self->sendMonitorTrap(
-                               $target,
-                               $Specific_Trap_Type,
-                               $type,
-                               $threshold,
-                               $name,
-                               $ds);
-    } elsif ($alarmType eq 'FILE') {
-        $self->LogToFile($alarmArgs->[0],'ADD',$name,$ds);
-    } else {
-        Warn("Alarm Type $alarmType is not known.");
-    }
-}
+    my $return = \&dispatchAlarm(\@_, $action);
+    return;
+};
 
-# Action to clear an alarm, you can change this to do whatever you need
-# Default is to send a trap
+# Action to clear an alarm
 sub Clear {
-    my($self,$target,$name,$ds,$type,$threshold,$alarmType,$alarmArgs,$val) = @_;
+# order of arguments: $self, $target, $name, $ds, $type, $threshold,
+#                     $alarmType, $alarmArgs, $val
+    my $action = 'CLEAR';
 
-    if ($alarmType eq 'EXEC') {
-        system($alarmArgs->[1]) ;
-        Info("Cleared event with shell command '".$alarmArgs->[1]."' .");
-    } elsif ($alarmType eq 'MAIL') {
-        $self->sendEmail(
-                         'clear',
-                         $alarmArgs,
-                         $type,
-                         $threshold,
-                         $name,
-                         $ds,
-                         $val,
-                         $target->{'inst'});
-    } elsif ($alarmType eq 'FUNC') {
-        if (defined $main::gMonFuncEnabled) {
-            eval($alarmArgs->[1]);
-            Info("Cleared event with FUNC '".$alarmArgs->[1]."' .");
-        } else {
-            Warn("Exec monitor $threshold triggered, but executable alarms are not enabled.");
-        }
-    } elsif ($alarmType eq 'SNMP') {
-        my($Specific_Trap_Type) = 5; # Clear Trap
-        $self->sendMonitorTrap(
-                               $target,
-                               $Specific_Trap_Type,
-                               $type,
-                               $threshold,
-                               $name,
-                               $ds );
-    } elsif ($alarmType eq 'FILE') {
-        $self->LogToFile($alarmArgs->[0],'CLEAR',$name,$ds);
-    } else {
-        Warn("Alarm Type $alarmType is not known.");
+    my $return = \&dispatchAlarm(\@_, $action);
+    return;
+};
+
+sub alarmExec {
+    my ($args, $action) = @_;
+    my $alarmArgs       = $$args[7];
+    system($alarmArgs->[0]);
+
+    if ($action eq 'ADD') {
+        Info("Monitor: Triggered event with system command '".
+             $alarmArgs->[0]."' .");
     }
-}
+
+    else {
+        Info("Monitor: Cleared event with shell command '".
+             $alarmArgs->[1]."' .");
+    }
+
+    return;
+};
+
+sub alarmFile {
+    my ($args, $action)  = @_;
+    my ($self, $target)    = ($$args[0], $$args[1]);
+    my ($name, $ds)        = ($$args[2], $$args[3]);
+    my ($type, $threshold) = ($$args[4], $$args[5]);
+    my ($alarmArgs, $val)  = ($$args[7], $$args[8]);
+
+    $self->LogToFile($alarmArgs, $action, $name, $ds, $val);
+    return;
+};
+
+sub alarmFunc {
+    my ($args, $action) = @_;
+    my $alarmArgs       = $$args[7];
+
+    if (defined $main::gMonFuncEnabled) {
+
+        if ($action eq 'ADD') {
+            eval($alarmArgs->[0]);
+            Info("Monitor: Triggered event with FUNC '".$alarmArgs->[0]."' .");
+        }
+
+        elsif ($action eq 'CLEAR') {
+            eval($alarmArgs->[1]);
+            Info("Monitor: Cleared event with FUNC '".$alarmArgs->[1]."' .");
+        }
+
+    }
+
+    else {
+        Warn("Monitor: Exec triggered, but executable alarms are not enabled.");
+    }
+
+    return;
+};
+
+sub alarmMail {
+    my ($args, $action)    = @_;
+    my ($self, $target)    = ($$args[0], $$args[1]);
+    my ($name, $ds)        = ($$args[2], $$args[3]);
+    my ($type, $threshold) = ($$args[4], $$args[5]);
+    my ($alarmArgs, $val)  = ($$args[7], $$args[8]);
+    $self->sendEmail( $action, $alarmArgs, $type, $threshold,
+                      $name, $ds, $val, $target->{'inst'} );
+    return;
+};
+
+sub alarmSnmp {
+    my ($args, $action)    = @_;
+    my ($self, $target)    = ($$args[0], $$args[1]);
+    my ($name, $ds)        = ($$args[2], $$args[3]);
+    my ($type, $threshold) = ($$args[4], $$args[5]);
+    my ($val)              = ($$args[8]);
+
+    my $Specific_Trap_Type = $action eq 'ADD' ? 4 : 5;
+    $self->sendMonitorTrap( $target, $Specific_Trap_Type, $type,
+                            $threshold, $name, $ds, $val );
+    return;
+};
+
+sub alarmMeta {
+    my ($args, $action)    = @_;
+
+    # No action is required.
+    # Alarm data is already stored in the Cricket meta files by HandleTarget.pm
+    # The meta data is stored in $CRICKET/cricket-data/ directories.
+    return;
+};
 
 # Attempt to send an alarm trap for a given target
 sub sendMonitorTrap {
-    my($self,$target,$spec,$type,$threshold,$name,$ds) = @_;
+    my($self,$target,$spec,$type,$threshold,$name,$ds,$val) = @_;
 
     my $to = $target -> {'trap-address'};
     if (!defined($to)) {
-        Warn("No trap address defined for $target, couldn't send trap.");
-        Info("Threshold Failed: $threshold for target $target");
+        Warn("Monitor: No trap address defined for $target, couldn't send trap.");
+        Info("Monitor: Threshold Failed: $threshold for target $target");
         return;
     }
 
@@ -550,40 +652,47 @@ sub sendMonitorTrap {
             push(@VarBinds, "${OID_Prefix}.8", 'string', $1);
         }
     }
+    push(@VarBinds, "${OID_Prefix}.9", 'string', $val) unless (!defined($val));
 
-    Info("Trap Sent to $to:\n ". join(' -- ',@VarBinds));
+    Info("Monitor: Trap Sent to $to:\n ". join(' -- ',@VarBinds));
     snmpUtils::trap2($to,$spec,@VarBinds);
 }
 
 sub LogToFile {
-    my ($self, $filePath, $action, $targetName, $dataSourceName) = @_;
+    my ($self, $alarmArgs, $action, $targetName, $dataSourceName) = @_;
+
+    my $filePath = $alarmArgs->[0];
     my @lines = ();
     my $targetLine;
 
     return unless ($action eq 'ADD' || $action eq 'CLEAR');
 
-    # open for read or create if missing (so we open for append)
-    unless (open(INFILE, "+>>$filePath")) {
-        Info("Failed to open file $filePath");
-        return;
-    }
-
+    # try to open for read first and hunt for duplicate lines
     my $bFound = 0;
-    $targetLine = $targetName . " " . $dataSourceName;
-    while (<INFILE>) {
-        chomp;
-        if ($_ eq $targetLine) {
-            $bFound = 1;
-            # nothing to add
-            last if ($action eq 'ADD');
-        } else {
-            push (@lines, $_) if ($action eq 'CLEAR');
+
+    if (open(INFILE, "$filePath")) {
+        $targetLine = $targetName . " " . $dataSourceName;
+        while (<INFILE>) {
+            chomp;
+            if ($_ eq $targetLine) {
+                $bFound = 1;
+                # nothing to add
+                last if ($action eq 'ADD');
+            } else {
+                push (@lines, $_) if ($action eq 'CLEAR');
+            }
         }
     }
 
+    unless (open(INFILE, "+>>$filePath")) {
+        Info("Monitor: Failed to open file $filePath: $!");
+        return;
+    }
+
+
     # append the new line to the end of the file
     if ($action eq 'ADD' && $bFound == 0) {
-        Info("Appending line $targetLine to $filePath");
+        Info("Monitor: Appending line $targetLine to $filePath");
         print INFILE $targetLine . "\n";
         close(INFILE);
         return;
@@ -593,22 +702,23 @@ sub LogToFile {
     close(INFILE);
 
     if ($action eq 'ADD' && $bFound == 1) {
-        Info("$targetName $dataSourceName already in file $filePath");
+        Info("Monitor: $targetName datasource $dataSourceName already in ".
+             "file $filePath");
         return;
     }
 
     if ($action eq 'CLEAR' && $bFound == 0) {
-        Info("$targetName $dataSourceName already deleted from file $filePath");
+        Info("Monitor: $targetName datasource $dataSourceName already " .
+             "deleted from file $filePath");
         return;
     }
-
     # need to print out @lines, which excludes the $targetLine
     # overwrite old file
     unless (open(OUTFILE, ">$filePath")) {
-        Info("Failed to open file $filePath");
+        Info("Monitor: Failed to open file $filePath: $!");
         return;
     }
-    Info("Deleting $targetLine from $filePath");
+    Info("Monitor: Deleting $targetLine from $filePath");
     print OUTFILE join("\n", @lines);
     print OUTFILE "\n" unless (scalar(@lines) == 0);
     close(OUTFILE);
@@ -619,7 +729,8 @@ sub sendEmail {
 
     my $to = $alarmArgs -> [1];
     if (!defined($to)) {
-        Warn("No destination address defined for $target, couldn't send email.");
+        Warn("Monitor: No destination address defined for $target, " .
+             "couldn't send email.");
         return;
     }
 
@@ -638,15 +749,15 @@ sub sendEmail {
 
     my $mail_program = $alarmArgs -> [0];
     if (!defined($mail_program)) {
-        Warn("No email-program defined. Not sending email");
+        Warn("Monitor: No email-program defined. Not sending email");
         return;
     } elsif (!open(MAIL, "|$mail_program -s 'Cricket $spec: $target' $to\n")) {
-        Warn("Failed to open pipe to mail program");
+        Warn("Monitor: Failed to open pipe to mail program: $!");
         return;
     }
     Debug("|$mail_program -s 'Cricket $spec: $target' $to\n");
     print (MAIL join ("\n", @Message));
-    Info("Email sent to: $to\n" . join(' -- ', @Message));
+    Info("Monitor: Email sent to: $to\n" . join(' -- ', @Message));
     close(MAIL);
 }
 1;
